/* ASN.1 DER encode/decode */

#pragma	lib	"libder.a"
#pragma	src	"/sys/src/libder"

typedef struct Elem Elem;
typedef struct Tag Tag;
typedef struct Value Value;
typedef struct Bytes Bytes;
typedef struct Ints Ints;
typedef struct Bits Bits;
typedef struct Elist Elist;

enum{		/* tag classes */
	Universal			= 0x00,
	Application			= 0x40,
	Context				= 0x80,
	Private				= 0xc0,
};

enum {		/* universal tags */
	EOC					= 0,
	BOOLEAN				= 1,
	INTEGER				= 2,
	BIT_STRING			= 3,
	OCTET_STRING		= 4,
	NULLTAG				= 5,
	OBJECT_ID			= 6,
	ObjectDescriptor	= 7,
	EXTERNAL			= 8,
	REAL				= 9,
	ENUMERATED			= 10,
	EMBEDDED_PDV		= 11,
	UTF8String			= 12,
	SEQUENCE			= 16,		/* also SEQUENCE OF */
	SETOF				= 17,		/* also SETOF OF */
	NumericString		= 18,
	PrintableString		= 19,
	TeletexString		= 20,
	VideotexString		= 21,
	IA5String			= 22,
	UTCTime				= 23,
	GeneralizedTime		= 24,
	GraphicString		= 25,
	VisibleString		= 26,
	GeneralString		= 27,
	UniversalString		= 28,
	BMPString			= 30,
 };


enum {
	TAG_MASK 			= 0x1F,
	CONSTR_MASK			= 0x20,
	CLASS_MASK			= 0xC0,
	MAXOBJIDLEN			= 20,
};

/* decoding errors */
enum {
	ASN_OK,
	ASN_ESHORT,
	ASN_ETOOBIG,
	ASN_EVALLEN,
	ASN_ECONSTR,
	ASN_EPRIM,
	ASN_EINVAL,
	ASN_EUNIMPL
};

struct Bytes {
	int	len;
	uchar	data[1];
};

struct Ints {
	int	len;
	int	data[];
};

struct Bits {
	int	len;		/* numder of bytes */
	int	unusedbits;	/* unused bits in last byte */
	uchar	data[];		/* most-significant bit first */
};

struct Tag {
	int	class;
	int	num;
};

enum { VBool, VInt, VOctets, VBigInt, VReal, VOther,
	VBitString, VNull, VEOC, VObjId, VString, VSeq, VSet
};

struct Value {
	int	tag;		/* VBool, etc. */
	union {
		int	boolval;
		int	intval;
		int	enumval;
		Bytes*	octetsval;
		Bytes*	bigintval;
		Bytes*	realval;	/* undecoded; hardly ever used */
		Bytes*	otherval;
		Bits*	bitstringval;
		Ints*	objidval;
		char*	stringval;
		Elist*	seqval;
		Elist*	setval;
	} u;  /* (Don't use anonymous unions, for ease of porting) */
};

struct Elem {
	Tag	tag;
	Value	val;
};

struct Elist {
	Elist*	tl;
	Elem	hd;
};

/* alloc.c */
void *der_malloc(int n);
char *der_strdup(char *s);

/* decode.c */
int der_tag_decode(uchar **pp, uchar *pend, Tag *ptag, int *pisconstr);		/* for libsec/x509.c only */
int der_len_decode(uchar **pp, uchar *pend, int *plength);					/* for libsec/x509.c only */
int der_do_decode(uchar **pp, uchar *pend, Elem *pelem);					/* for libsec/x509.c only */
int der_decode(uchar *a, int alen, Elem *pelem);

/* dump.c */
int der_dump(uchar *der, int len);

/* encode.c */
int der_encode(Elem e, Bytes **pbytes);

/* is.c */
int is_seq(Elem *pe, Elist **pseq);
int is_appl(Elem *pe, int *api, Elist **pseq);
int is_set(Elem *pe, Elist **pset);
int is_int(Elem *pe, int *pint);
int is_bigint(Elem *pe, Bytes **pbigint);
int is_bitstring(Elem *pe, Bits **pbits);
int is_octetstring(Elem *pe, Bytes **poctets);
int is_oid(Elem *pe, Ints **poid);
int is_string(Elem *pe, char **pstring);
int is_time(Elem *pe, char **ptime);

/* mk.c */
Elem Null(void);
Elem mkint(int j);
Elem mkbool(int tf);
Elem mkenum(int n);
Elem mkstr(char *s);
Elem mkutf8(char *s);
Elem mkoctet(uchar *buf, int buflen);
Elem mkbits(uchar *buf, int buflen);
Elem mkutc(long t);
Elem mkoid(Ints *oid);
Elem mkseq(Elist *el);
Elem mkset(Elist *el);
Elem mkname(Ints *oid, char *subj, int stype);
Elem mkDN(char *dn);
Elem mkcont(Elem e, int num);
Elem mkcontseq(int id, Elist *el);
Elem mkaltname(char *s);
Elem mkcontstr(int id, char *s);
Elem mkappl(int id, Elist *el);

/* new.c */
Bytes *newbytes(int len);
Bytes *makebytes(uchar *buf, int len);
void freebytes(Bytes *b);
Ints *newints(int len);
Ints *makeints(int *buf, int len);
void freeints(Ints *b);
Bits *newbits(int len);
Bits *makebits(uchar *buf, int len, int unusedbits);
void freebits(Bits *b);
Elist *mkel(Elem e, Elist *tail);
int elistlen(Elist *el);
void freeelist(Elist *el);
void freevalfields(Value *v);


extern char *Asn1Errstr[];
