#include <u.h>
#include <libc.h>
#include "der.h"

enum {
	DirectoryString = 0,
};

static struct Ints7pref {
	int len;
	int	data[7];
	char prefix[4];
	int stype;
} DN_oid[] = {
	{ 4, 2, 5, 4, 6, 0, 0, 0,					"C=", 	PrintableString},
	{ 4, 2, 5, 4, 8, 0, 0, 0,					"ST=",	DirectoryString},
	{ 4, 2, 5, 4, 7, 0, 0, 0,					"L=", 	DirectoryString},
	{ 4, 2, 5, 4, 10, 0, 0, 0,					"O=", 	DirectoryString},
	{ 4, 2, 5, 4, 11, 0, 0, 0,					"OU=",	DirectoryString},
	{ 4, 2, 5, 4, 3, 0, 0, 0,					"CN=",	DirectoryString},
	{ 7, 1, 2, 840, 113549, 1, 9, 1,			"E=",	IA5String},
	{ 7, 0, 9, 2342, 19200300, 100, 1, 25,		"DC=",	IA5String},
};

Elem
Null(void)
{
	Elem e;

	e.tag.class = Universal;
	e.tag.num = NULLTAG;
	e.val.tag = VNull;
	return e;
}

Elem
mkint(int j)
{
	Elem e;

	e.tag.class = Universal;
	e.tag.num = INTEGER;
	e.val.tag = VInt;
	e.val.u.intval = j;
	return e;
}

Elem
mkbool(int tf)
{
	Elem e;

	e.tag.class = Universal;
	e.tag.num = BOOLEAN;
	e.val.tag = VBool;
	e.val.u.intval = tf;
	return e;
}

Elem
mkenum(int n)
{
	Elem e;

	e.tag.class = Universal;
	e.tag.num = ENUMERATED;
	e.val.tag = VInt;
	e.val.u.intval = n;
	return e;
}

Elem
mkutf8(char *s)
{
	Elem e;

	e.tag.class = Universal;
	e.tag.num = UTF8String;
	e.val.tag = VString;
	e.val.u.stringval = der_strdup(s);
	return e;
}

Elem
mkstr(char *s)
{
	Elem e;

	e.tag.class = Universal;
	e.tag.num = PrintableString;
	e.val.tag = VString;
	e.val.u.stringval = der_strdup(s);
	return e;
}


Elem
mkoctet(uchar *buf, int buflen)
{
	Elem e;

	e.tag.class = Universal;
	e.tag.num = OCTET_STRING;
	e.val.tag = VOctets;
	e.val.u.octetsval = makebytes(buf, buflen);
	return e;
}

Elem
mkbits(uchar *buf, int buflen)
{
	Elem e;

	e.tag.class = Universal;
	e.tag.num = BIT_STRING;
	e.val.tag = VBitString;
	e.val.u.bitstringval = makebits(buf, buflen, 0);
	return e;
}

Elem
mkutc(long t)
{
	Elem e;
	char utc[50];
	Tm *tm = gmtime(t);

	e.tag.class = Universal;
	e.tag.num = UTCTime;
	e.val.tag = VString;
	snprint(utc, sizeof(utc), "%.2d%.2d%.2d%.2d%.2d%.2dZ",
		tm->year % 100, tm->mon+1, tm->mday, tm->hour, tm->min, tm->sec);
	e.val.u.stringval = der_strdup(utc);
	return e;
}

Elem
mkoid(Ints *oid)
{
	Elem e;

	e.tag.class = Universal;
	e.tag.num = OBJECT_ID;
	e.val.tag = VObjId;
	e.val.u.objidval = makeints(oid->data, oid->len);
	return e;
}

Elem
mkseq(Elist *el)
{
	Elem e;

	e.tag.class = Universal;
	e.tag.num = SEQUENCE;
	e.val.tag = VSeq;
	e.val.u.seqval = el;
	return e;
}

Elem
mkset(Elist *el)
{
	Elem e;

	e.tag.class = Universal;
	e.tag.num = SETOF;
	e.val.tag = VSet;
	e.val.u.setval = el;
	return e;
}

static int
is_printable(char *s)
{
	int c;

	while((c = (uchar)*s++) != 0){
		if((c >= 'a' && c <= 'z')
		|| (c >= 'A' && c <= 'Z')
		|| (c >= '0' && c <= '9')
		|| strchr("'=()+,-./:? ", c) != nil)
			continue;
		return 0;
	}
	return 1;
}

Elem
mkname(Ints *oid, char *subj, int stype)
{
	if(stype == DirectoryString && is_printable(subj))
		return mkset(mkel(mkseq(mkel(mkoid(oid), mkel(mkstr(subj), nil))), nil));
	else
		return mkset(mkel(mkseq(mkel(mkoid(oid), mkel(mkutf8(subj), nil))), nil));
}


Elem
mkDN(char *dn)
{
	Elist* el;
	int i, j, nf;
	char *f[20], *prefix, *d2;

	el = nil;
	d2 = der_strdup(dn);
	nf = tokenize(d2, f, nelem(f));
	for(i=nf-1; i>=0; i--){
		for(j=0; j<nelem(DN_oid); j++){
			prefix = DN_oid[j].prefix;
			if(strncmp(f[i],prefix, strlen(prefix))==0){
				el = mkel(mkname((Ints *)DN_oid[j].data, f[i]+strlen(prefix), DN_oid[j].stype), el);
				break;
			}
		}
	}
	free(d2);
	return mkseq(el);
}

Elem
mkcont(Elem e, int num)
{
	e = mkseq(mkel(e, nil));
	e.tag.class = Context;
	e.tag.num = num;
	return e;
}

Elem
mkcontstr(int id, char *s)
{
	Elem e;

	e.tag.class = Context;
	e.tag.num = id;
	e.val.tag = VString;
	e.val.u.stringval = der_strdup(s);
	return e;
}

Elem
mkcontseq(int id, Elist *el)
{
	Elem e;

	e.tag.class = Context;
	e.tag.num = id;
	e.val.tag = VSeq;
	e.val.u.seqval = el;
	return e;
}

Elem
mkaltname(char *s)
{
	Elem e;
	int i;

	for(i=0; i<nelem(DN_oid); i++){
		if(strstr(s, DN_oid[i].prefix) != nil)
			return mkcont(mkDN(s), 4); 				/* DN */
	}
	e = mkutf8(s);
	e.tag.class = Context;
	e.tag.num = strchr(s, '@') != nil ? 1 : 2; 		/* email : DNS */
	return e;
}

Elem
mkappl(int id, Elist *el)
{
	Elem e;

	e.tag.class = Application;
	e.tag.num = id;
	e.val.tag = VSeq;
	e.val.u.seqval = el;
	return e;
}

